

// From app.json

var DATA_CONFIG = 'imatch.DBSearch.config';
const  DATA_CONFIG_VERSION = 1

var config = {
    version : DATA_CONFIG_VERSION,
    scope: 'db',
    fwMode: 'fwm-result'
}

// We set this flag for the "progress button"
var running = false;

// We use a fairly large page size.
// This speeds up processing a lot for large databases.
var pageSize = 5000;

var currentPage = 0;
var searchScope = '';
var filesTotal = 0;
var filesWindow = 0;
var filesRange = 0;
var resultWindowInstanceId = 0;
var matches = [];
var sourceFiles = [];
var bulkMatchFileNames = [];
var bulkMatchNameOnly = false;

// Hide the "progress button" initially
$('#btn-running').hide();

$(document).ready(function () {

    // Get the info about the current user.
    // Then load the stored configuration data.
    IMatch.userInfo().then(function(response) {
        DATA_CONFIG = DATA_CONFIG.replace('{u}',response.userName);
        IMWS.dataGet({name: DATA_CONFIG}).then(function(response) {
            var data = JSON.parse(response.value);
            config = data;

        }).always(function() {
            // Whether or not we had stored configuration data,
            // initialize the controls
            $('#digit-match-length').val(config.digitMatch);
            $('#sim-max-distance').val(config.simMatch);
            $('#search-scope option[value="' + config.scope + '"]').prop('selected', true);


        });
    });

    IMatchTranslate.init(IMatchTranslate.getUserLanguage(),'en','res/');

// Get the total files in the current scope/database for use later in the results header
    IMatch.get(`info`).then(function(imwsInfo) { 
	  filesTotal = imwsInfo.database.files;
	});

    IMWS.get('v1/files',{
                    idlist: IMatch.idlist.fileWindowFiles,
                    fields: 'id'
                }).then(function(response) {
                    filesWindow = response.files.length;
					console.log("northern:" + filesWindow);
                });

    setupMonitoring();

    // Bind button handlers

    $('#btn-all-match').click(function(e) {
        if (!running) {
            runSearch();
			$('#btn-running').show();
        }
        e.preventDefault();
        return false;
    });


    $('#search-scope').on('change',event => {
        config.scope = $('#search-scope').val();
        if (config.scope == 'db') {
            $('#fw-mode-group').hide();
        }
        else {
            $('#fw-mode-group').show();
        }
    });


  $('#btn-find').click(function(e) {
  
   var taggerS = $('#tag-pattern').val();
        if ($('#tag-pattern').val() != '') {
			console.log("made it inside: " + taggerS);
            IMWS.get('v1/metadata/tags',{
                pattern: taggerS,
                grouppattern: 'XMP',
                searchkey: true
            }).then(function(response) {
				
				console.log("results: " + JSON.stringify(response.tags, null, 2));

            }, function(error) {
            console.log ("we erred: "+JSON.stringify(error, null, 2));
        })
        } 
});

});


///////////////////////////////////////////////////////////////////
// We monitor IMatch events.
function setupMonitoring()
{
    if (IMatch.isEmbedded()) {
        IMatch.openWebSocket();
    }
    else {
        // Running in the browser
        IMatch.openWebSocket(IMatch.DEFAULT_WS_URL);
    }

    // Monitor IMatch events to get notice about file window events.
    IMatch.webSocket().onmessage = function(e) {
        var msg = JSON.parse(e.data);
		// check to see if the scope changed - if it did, update the total number of files in the window
		if (msg.msg == 'fileWindow.scopeChanged') {
			    IMWS.get('v1/files',{
                    idlist: IMatch.idlist.fileWindowFiles,
                    fields: 'id'
                }).then(function(response) {
                    filesWindow = response.files.length;
                });
		}
		
        if (msg.msg == 'app.appfilter.removed') {
            // An app filter was removed. This may be the
            // filter we have set, or another one.
            if (msg.data.appId == APP_ID) {
                // Our filter. Remove the clear filter button.
                $('#btn-clear-filter').hide();
            }
        }
    }
}

///////////////////////////////////////////////////////////////////
// Save the configuration batck
function convertString()
{
config.allMatchPattern = $('#all-match-pattern').val();

//console.log ("match: " + config.allMatchPattern);
config.allMatchPattern = config.allMatchPattern.replace(/\+/g,"AND ");
config.allMatchPattern = config.allMatchPattern.replace(/\-/g,"OR ");
config.allMatchPattern = config.allMatchPattern.replace(/\|/g,"AND NOT ");
//console.log ("match after: " + config.allMatchPattern);
}




///////////////////////////////////////////////////////////////////
// Save the configuration batck
function saveConfig()
{
    config.version = DATA_CONFIG_VERSION;
    config.scope = $('#search-scope').val();

    IMWS.dataSet({
        name: DATA_CONFIG,
        value: JSON.stringify(config)
    });
}


///////////////////////////////////////////////////////////////////
// Open or update a result window to show the results.
function openResultWindow(title)
{
	//console.log("Configuration: " + JSON.stringify(config, null, 2));
    if (config.reuseResultWindow && (resultWindowInstanceId != 0)) {
        // Reuse the existing result window if it still exists (may have been closed by the user).
        // We also update it when nothing was found!
        IMatch.updateResultWindow({
            instanceid: resultWindowInstanceId,
            name: (title === undefined) ? '' : title,
            id: matches.join(',')
        }).then(function(response) {
            // Success
        },error => {
            // Does not exist anymore. We call this again to open a new window.
            resultWindowInstanceId = 0;
            $('#btn-reuse').attr('disabled',true );
            openResultWindow(title);
        });
    }
    else {
		//console.log("in here: " + JSON.stringify(matches, null, 2));
        IMatch.openResultWindow({
            name: (title === undefined) ? IMatchTranslate.getStr('T_RESULT_WINDOW_NAME') : title,
            id: matches.join(',')
        }).then(function(response) {
            resultWindowInstanceId = response.instanceId;

	});
    }
}


///////////////////////////////////////////////////////////////////
// Search for metadata across all tags
//   TODO: add drop down of tags to restrict to
function runSearch()
{
    config.mode = 4;
    saveConfig();
	running = true;

    convertString();
    if (config.allMatchPattern.length == 0) return;

    // We either search the entire database, or search the active file window.
    IMatch.get('v1/search/metadata',{
        scope: $('#search-scope').val() == 'db' ? 'database' : 'idlist',
        idlist: IMatch.idlist.fileWindowFilesTotal,
        pattern: config.allMatchPattern,
        advancedmode: true,
        //tags: "hierarchicalkeywords",
        fields: 'id',
    }).then(function(response) {

  if ($('#search-scope').val() == 'db') {
		  filesRange = filesTotal;
	}
      else { 
	      filesRange =filesWindow;
	  }
				//console.log("filesRange: " + filesRange + "filesTotal: " + filesTotal + "filesWindow: " + filesWindow);

        matches = response.files;
        $('#btn-running').hide();
        complete(true, "Found: [" + matches.length + "/" + filesRange + "] -> Search: " + config.allMatchPattern);

    },function(error) {
        complete(false);
    });
}



///////////////////////////////////////////////////////////////////
// This is called when all files have been processed or
// when the app runs into an error.
function complete(success,title) {
    running = false;

    if (success) {

        if ((config.scope == 'fw') && (config.fwMode == 'fwm-filter')) {
            // Filter the active file window. We include both the matches and the currently selected files!
            if (matches.length > 0) {
                IMatch.get('v1/files',{
                    idlist: IMatch.idlist.fileWindowSelection,
                    fields: 'id'
                }).then(function(response) {
                    // merge the selected files with the matches
                    for (let f of response.files) {
                        if (matches.indexOf(f) < 0) {
                            matches.push(f);
                        }
                    }
                    IMatch.fileWindowFilterUpdate({
                        appid: IMatch.appId() != undefined ? IMatch.appId() : APP_ID,
                        mode: 'include',
                        id: matches.join(',')
                    });
                    $('#btn-clear-filter').fadeIn();
                });
            };
        }
        else {
            // Show results in a result window
            if (matches.length > 0) {
                openResultWindow(title);
            }
        }
    }
}

